/*
 * tclAppInit.c,v 1.15 2002/02/21 23:07:49 joe Exp
 *
 * Tcl_AppInit() and main() routines for costsh and costwish.
 * If -DUSE_TK is defined, calls Tk_Main (for costwish);
 * otherwise calls Tcl_Main (for costsh).
 *
 * Author: Joe English <jenglish@flightlab.com>
 */

#ifndef USE_TK
#define USE_TK 0
#endif

#include <stdlib.h>
#include <stdio.h>
#include "tcl.h"
#include "project.h"

#if USE_TK
#	include "tk.h"
#endif

extern int Cost_Init(Tcl_Interp *);

static char init_script[] = "cost::commandLine";

/* If we're in batch mode (sgmls ... | costsh ...)
 * and 'cost::commandLine' (or any other initialization) fails,
 * there's a good chance that stdin has not yet been consumed.
 * Since we _definitely_ don't want to feed sgmls' output to
 * Tcl, we exit immediately if an initialization error occurs
 * and this is not an interactive shell.
 *
 * errorInfo traceback goes to stdout instead of stderr
 * so it can be redirected to a file; main() prints a shorter
 * message on stderr.
 */
static char error_script[] =
    "puts stdout $errorInfo; if {$tcl_interactive == 0} { exit 1 }\n";

int Tcl_AppInit(Tcl_Interp *interp)
{
    int status = TCL_OK;

    if ((status = Tcl_Init(interp)) != TCL_OK) goto error;
#if USE_TK
    if ((status = Tk_Init(interp)) != TCL_OK) goto error;
#endif

    /*
     * Initialize Cost:
     */
#if HARDCODE_PKGDIR
    /* Set default value for COSTLIB, so user doesn't have
     * to specify it in the environment:
     */
    Tcl_SetVar(interp, "COSTLIB", PKGDIR, TCL_GLOBAL_ONLY);
#endif
    if ((status = Cost_Init(interp)) != TCL_OK)
	goto error;

    if ((status = Tcl_Eval(interp, init_script)) != TCL_OK)
	goto error;

    return status;

error:
    {
	Tcl_Channel errChannel = Tcl_GetStdChannel(TCL_STDERR);
	if (errChannel) {
	    Tcl_Write(errChannel, "COST initialization failure:\n", -1);
	    Tcl_Write(errChannel, interp->result, -1);
	    Tcl_Write(errChannel, "\n", 1);
	} else {
	    fprintf(stderr, "COST initialization failure: %s\n",interp->result);
	}
    }
    Tcl_Eval(interp, error_script);
    return status;
}

int main(int argc, char *argv[])
{
#if USE_TK
    Tk_Main(argc, argv, Tcl_AppInit);
#else
    Tcl_Main(argc, argv, Tcl_AppInit);
#endif
    return 0;
}
/*EOF*/
